"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useFaceCaptchaContext = exports.default = exports.SCREEN = exports.FaceCaptchaProvider = void 0;
var _index = require("../index");
var _react = _interopRequireWildcard(require("react"));
var _permissions = require("../utils/permissions");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const SCREEN = Object.freeze({
  INSTRUCTION_VIEW: 1,
  PERMISSION_VIEW: 2
});
exports.SCREEN = SCREEN;
const FaceCaptchaContext = /*#__PURE__*/(0, _react.createContext)(undefined);
const FaceCaptchaProvider = _ref => {
  let {
    children,
    onError,
    onSuccess
  } = _ref;
  const [screen, setScreen] = (0, _react.useState)(1);
  const [navigation, setNavigation] = (0, _react.useState)();
  const [options, setOptions] = (0, _react.useState)();
  const [callbackView, setCallbackView] = (0, _react.useState)('');
  const onFaceCaptchaError = error => {
    onError(error);
  };
  const onFaceCaptchaSuccess = result => {
    onSuccess(result);
  };
  function onBack() {
    switch (screen) {
      case SCREEN.INSTRUCTION_VIEW:
        navigation.goBack(null);
        onFaceCaptchaError({
          code: '0',
          message: 'RESULT_CANCELED'
        });
        break;
      case SCREEN.PERMISSION_VIEW:
        setScreen(1);
        break;
    }
  }
  function startLiveness2d() {
    (0, _index.startFaceCaptcha)(options).then(result => {
      onSuccess(result);
    }).catch(error => {
      onError(error);
    }).finally(() => {
      navigation.navigate(callbackView);
    });
  }
  function onAskPermission() {
    (0, _permissions.askPermission)().then(result => {
      result === true && startLiveness2d();
      result === false && setScreen(1);
    });
  }
  const contextValue = {
    screen,
    setScreen,
    navigation,
    setNavigation,
    onFaceCaptchaError,
    onFaceCaptchaSuccess,
    onBack,
    options,
    setOptions,
    startLiveness2d,
    onAskPermission,
    callbackView,
    setCallbackView
  };
  return /*#__PURE__*/_react.default.createElement(FaceCaptchaContext.Provider, {
    value: contextValue
  }, children);
};
exports.FaceCaptchaProvider = FaceCaptchaProvider;
const useFaceCaptchaContext = () => {
  const context = (0, _react.useContext)(FaceCaptchaContext);
  if (!context) {
    throw new Error('useFaceCaptchaContext must be used within a FaceCaptchaProvider');
  }
  return context;
};
exports.useFaceCaptchaContext = useFaceCaptchaContext;
var _default = FaceCaptchaContext;
exports.default = _default;
//# sourceMappingURL=FaceCaptchaContext.js.map