import { startDocumentscopy } from '../index';
import React, { createContext, useContext, useState } from 'react';
import { askPermission } from '../utils/permissions';
export const SCREEN = Object.freeze({
  INSTRUCTION_VIEW: 1,
  PERMISSION_VIEW: 2
});
const DocCoreContext = /*#__PURE__*/createContext(undefined);
export const DocCoreProvider = _ref => {
  let {
    children,
    onError,
    onSuccess
  } = _ref;
  const [screen, setScreen] = useState(1);
  const [navigation, setNavigation] = useState();
  const [options, setOptions] = useState();
  const [callbackView, setCallbackView] = useState('');
  const onDocCoreError = error => {
    onError(error);
  };
  const onDocCoreSuccess = result => {
    onSuccess(result);
  };
  function onBack() {
    switch (screen) {
      case SCREEN.INSTRUCTION_VIEW:
        navigation.goBack(null);
        onDocCoreError({
          code: '0',
          message: 'RESULT_CANCELED'
        });
        break;
      case SCREEN.PERMISSION_VIEW:
        setScreen(1);
        break;
    }
  }
  function startDocCore() {
    startDocumentscopy(options).then(result => {
      onSuccess(result);
    }).catch(error => {
      onError(error);
    }).finally(() => {
      navigation.navigate(callbackView);
    });
  }
  function onAskPermission() {
    askPermission().then(result => {
      result === true ? startDocCore() : setScreen(1);
    }).finally(() => {
      setScreen(1);
    });
  }
  const contextValue = {
    screen,
    setScreen,
    navigation,
    setNavigation,
    onDocCoreError,
    onDocCoreSuccess,
    onBack,
    options,
    setOptions,
    startDocCore,
    onAskPermission,
    callbackView,
    setCallbackView
  };
  return /*#__PURE__*/React.createElement(DocCoreContext.Provider, {
    value: contextValue
  }, children);
};
export const useDocCoreContext = () => {
  const context = useContext(DocCoreContext);
  if (!context) {
    throw new Error('useDocCoreContext must be used within a DocCoreProvider');
  }
  return context;
};
export default DocCoreContext;
//# sourceMappingURL=DocCoreContext.js.map