import { startFaceCaptcha } from '../index';
import React, { createContext, useContext, useState } from 'react';
import { askPermission } from '../utils/permissions';
export const SCREEN = Object.freeze({
  INSTRUCTION_VIEW: 1,
  PERMISSION_VIEW: 2
});
const FaceCaptchaContext = /*#__PURE__*/createContext(undefined);
export const FaceCaptchaProvider = _ref => {
  let {
    children,
    onError,
    onSuccess
  } = _ref;
  const [screen, setScreen] = useState(1);
  const [navigation, setNavigation] = useState();
  const [options, setOptions] = useState();
  const [callbackView, setCallbackView] = useState('');
  const onFaceCaptchaError = error => {
    onError(error);
  };
  const onFaceCaptchaSuccess = result => {
    onSuccess(result);
  };
  function onBack() {
    switch (screen) {
      case SCREEN.INSTRUCTION_VIEW:
        navigation.goBack(null);
        onFaceCaptchaError({
          code: '0',
          message: 'RESULT_CANCELED'
        });
        break;
      case SCREEN.PERMISSION_VIEW:
        setScreen(1);
        break;
    }
  }
  function startLiveness2d() {
    startFaceCaptcha(options).then(result => {
      onSuccess(result);
    }).catch(error => {
      onError(error);
    }).finally(() => {
      navigation.navigate(callbackView);
    });
  }
  function onAskPermission() {
    askPermission().then(result => {
      result === true && startLiveness2d();
      result === false && setScreen(1);
    });
  }
  const contextValue = {
    screen,
    setScreen,
    navigation,
    setNavigation,
    onFaceCaptchaError,
    onFaceCaptchaSuccess,
    onBack,
    options,
    setOptions,
    startLiveness2d,
    onAskPermission,
    callbackView,
    setCallbackView
  };
  return /*#__PURE__*/React.createElement(FaceCaptchaContext.Provider, {
    value: contextValue
  }, children);
};
export const useFaceCaptchaContext = () => {
  const context = useContext(FaceCaptchaContext);
  if (!context) {
    throw new Error('useFaceCaptchaContext must be used within a FaceCaptchaProvider');
  }
  return context;
};
export default FaceCaptchaContext;
//# sourceMappingURL=FaceCaptchaContext.js.map